export function createScatterPlot() {
  let width;
  let height;
  let data;
  let xValue;
  let margin;
  let radius;
  let xAxisLabel;
  let yAxisLabel;

  function chart(selection) {
    function renderChart() {
      const xScale = d3
        .scaleLinear()
        .domain([0, d3.max(data, xValue)])
        .range([margin.left, width - margin.right]);

      const yScale = d3
        .scaleLinear()
        .domain([0, d3.max(data, (d) => d.SalesVolume)])
        .range([height - margin.bottom, margin.top]);

      // Clear previous x-axis elements
      selection.selectAll('.x-axis').remove();

      // Add X-axis
      const xAxis = selection
        .append('g')
        .attr('class', 'x-axis')
        .attr(
          'transform',
          `translate(0, ${height - margin.bottom})`,
        )
        .call(d3.axisBottom(xScale))
        .selectAll('text')
        .style('text-anchor', 'end')
        .attr('dx', '-0.5em')
        .attr('dy', '0.15em')
        .attr('transform', 'rotate(-45)') // Rotate x-axis labels
        .attr(
          'transform',
          (d, i) => `translate(0, ${i % 2 === 0 ? 0 : 15})`,
        );

      // Add X-axis Label
      selection
        .append('text')
        .attr('class', 'x-axis-label')
        .attr('x', width / 2)
        .attr('y', height)
        .style('text-anchor', 'middle')
        .text(xAxisLabel);

      // Add Y-axis
      selection
        .append('g')
        .attr('transform', `translate(${margin.left},0)`)
        .call(d3.axisLeft(yScale))
        .append('text')
        .attr('class', 'y-axis-label')
        .attr('transform', 'rotate(-90)')
        .attr('x', -height / 2)
        .attr('y', -margin.left + 20)
        .style('text-anchor', 'middle')
        .text(yAxisLabel);

      // Add circles
      selection
        .selectAll('circle')
        .data(data)
        .join('circle')
        .attr('cx', (d) => xScale(xValue(d)))
        .attr('cy', (d) => yScale(d.SalesVolume))
        .attr('r', radius)
        .attr('fill', 'steelblue');
    }

    // Initial render
    renderChart();

    return chart;
  }

  chart.width = function (_) {
    return arguments.length ? ((width = _), chart) : width;
  };

  chart.height = function (_) {
    return arguments.length
      ? ((height = _), chart)
      : height;
  };

  chart.data = function (_) {
    return arguments.length ? ((data = _), chart) : data;
  };

  chart.xValue = function (_) {
    return arguments.length
      ? ((xValue = _), chart)
      : xValue;
  };

  chart.margin = function (_) {
    return arguments.length
      ? ((margin = _), chart)
      : margin;
  };

  chart.radius = function (_) {
    return arguments.length
      ? ((radius = _), chart)
      : radius;
  };

  chart.xAxisLabel = function (_) {
    return arguments.length
      ? ((xAxisLabel = _), chart)
      : xAxisLabel;
  };

  chart.yAxisLabel = function (_) {
    return arguments.length
      ? ((yAxisLabel = _), chart)
      : yAxisLabel;
  };

  return chart;
}
